//
//  CameraPlusIntegrationManager.m
//  Camera+
//
//  Created by Karl von Randow on 28/08/11.
//  Copyright 2011 Tap Tap Tap. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

////////////////////////////////////////////////////////////////////////////////////////////////////

/* Picker modes. Set these modes on the CameraPlusIntegrationManager to open the Camera+ picker in a
 * specific mode.
 */
typedef enum {
    
    /* Picker modes */
    CameraPlusIntegrationModeShootAndLightbox = 1,
    CameraPlusIntegrationModeShootOnly,
    CameraPlusIntegrationModeLightboxOnly,
 
    /* Edit mode is invoked with the special openCameraPlusEditorWithImage... selectors */
    CameraPlusIntegrationModeEdit,
    
    /* Share mode is invoked with the special openCameraPlusShareWithImage... selectors */
    CameraPlusIntegrationModeShare,
    CameraPlusIntegrationModeCreateWebLink
    
} CameraPlusIntegrationMode;

@class CameraPlusImages;
@class CameraPlusIntegrationResult;

#if NS_BLOCKS_AVAILABLE

typedef void (^CameraPlusIntegrationCompletionBlock)(CameraPlusIntegrationResult *result);
typedef void (^CameraPlusIntegrationCancelBlock)(CameraPlusIntegrationMode mode);

#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

@protocol CameraPlusIntegrationManagerDelegate;

@interface CameraPlusIntegrationManager : NSObject {
@private
    NSString *callbackURLScheme;
    NSInteger imageSize;
    BOOL autoCloseShareSuccess;
    CGFloat compressionQuality;
}


////////////////////////////////////////////////////////////////////////////////////////////////////

/* Access the shared CameraPlusIntegrationManager */

+ (CameraPlusIntegrationManager*)sharedManager;


/* Test whether Camera+ is installed on the device. */
@property (nonatomic, readonly) BOOL cameraPlusAvailable;

/* Test whether Camera+ with integration support is installed. If Camera+ is available, but the
 * integration isn't available, the user has an outdated version of Camera+ and could be invited
 * to upgrade.
 */
@property (nonatomic, readonly) BOOL cameraPlusIntegrationAvailable;

/* Set a url scheme for Camera+ to use to launch your app after using the picker. This url scheme
 * must be configured in your Info.plist. This can be a scheme, eg "my-app", or a partial
 * URL if you'd like to send back some state in the callback, eg. my-app://from-cameraplus or
 * my-app://from-cameraplus?name=value
 */
@property (nonatomic, retain) NSString *callbackURLScheme;

/* The maximum size in pixels for the longest side of the image. For example if you
 * set 1000 here the longest side of the image will be no bigger than 1000 pixels.
 * Aspect ratio is always maintained when resizing.
 * Defaults to 0 meaning no resizing.
 */
@property (nonatomic, assign) NSInteger imageSize;

/* Set to YES if you don't want Camera+ to show the user the successful share result
 * when using the CameraPlusIntegrationModeShare or CameraPlusIntegrationModeCreateWebLink modes.
 * Defaults to NO, and Camera+ will show the user the successful share (including the 
 * link to the shared photos). Camera+ will always return confirmation (and the link
 * in CameraPlusIntegrationModeCreateWebLink mode) back to your application in the success callback.
 */
@property (nonatomic, assign) BOOL autoCloseShareSuccess;

/* The JPEG quality to compress UIImages to for sending to Camera+. Defaults to 0.9 or
 * 90% JPEG quality.
 */
@property (nonatomic, assign) CGFloat compressionQuality;



/* Open the Camera+ picker */
- (BOOL)openCameraPlusPickerWithMode:(CameraPlusIntegrationMode)mode;


/* Open the Camera+ editor */
- (BOOL)openCameraPlusEditorWithImage:(UIImage*)image;
- (BOOL)openCameraPlusEditorWithImageData:(NSData*)imageData;


/* Open the Camera+ sharing */
- (BOOL)openCameraPlusSharingWithImage:(UIImage*)image;
- (BOOL)openCameraPlusSharingWithImageData:(NSData*)imageData;
- (BOOL)openCameraPlusSharingWithImages:(NSArray*)images;

- (BOOL)createWebLinkWithImage:(UIImage*)image;
- (BOOL)createWebLinkWithImageData:(NSData*)imageData;
- (BOOL)createWebLinkWithImages:(NSArray*)images;

////////////////////////////////////////////////////////////////////////////////////////////////////


/* Returns YES if the given url is a Camera+ picker callback, and should be handled by
 * the handleCameraPlusIntegrationCallback selector.
 */
- (BOOL)shouldHandleURLAsCameraPlusIntegrationCallback:(NSURL*)url;

/* Handle the callback from Camera+. This is invoked from the UIApplicationDelegate's
 * application:handleOpenURL:. Returns YES if the URL was handled, or NO if it was
 * not from Camera+.
 */
- (BOOL)handleCameraPlusIntegrationCallback:(NSURL*)url delegate:(id<CameraPlusIntegrationManagerDelegate>)delegate;

#if NS_BLOCKS_AVAILABLE
- (BOOL)handleCameraPlusIntegrationCallback:(NSURL*)url 
                            usingBlock:(CameraPlusIntegrationCompletionBlock)block
                           cancelBlock:(CameraPlusIntegrationCancelBlock)cancelBlock;
#endif

/* Utility method to create a dictionary compatible with the UIImagePickerControllerDelegate 
 * info dictionary.
 */
+ (NSDictionary*)imagePickerControllerInfoDictionaryForImage:(UIImage*)image metadata:(NSDictionary*)metadata;

@end



#pragma mark - Delegate

@protocol CameraPlusIntegrationManagerDelegate <NSObject>

@optional

- (void)cameraPlusIntegrationManager:(CameraPlusIntegrationManager*)manager
                    didReceiveImages:(CameraPlusImages*)images
                            withMode:(CameraPlusIntegrationMode)mode;

- (void)cameraPlusIntegrationManagerDidCancel:(CameraPlusIntegrationManager*)manager
                                     withMode:(CameraPlusIntegrationMode)mode;

- (void)cameraPlusIntegrationManagerDidShare:(CameraPlusIntegrationManager*)manager;

- (void)cameraPlusIntegrationManager:(CameraPlusIntegrationManager*)manager 
                    didCreateWebLink:(NSURL*)link;

@end

#pragma mark -

@interface CameraPlusIntegrationResult : NSObject {
@private
    CameraPlusImages *images;
    NSURL *link;
    CameraPlusIntegrationMode mode;
}

@property (nonatomic, retain) CameraPlusImages *images;
@property (nonatomic, retain) NSURL *link;
@property (nonatomic, assign) CameraPlusIntegrationMode mode;
@end


#pragma mark -

@interface CameraPlusImages : NSObject {
@private
    UIPasteboard *pasteboard;
    NSArray *images;
    NSArray *metadata;
}

@property (nonatomic, readonly) NSInteger numberOfImages;

- (UIImage *)imageAtIndex:(NSUInteger)index;
- (NSData *)imageDataAtIndex:(NSUInteger)index;
- (NSDictionary *)metadataAtIndex:(NSUInteger)index;

- (UIImage *)image;
- (NSArray *)images;

@end